==> opennms-34.0.2.tar.gz: opennms-34.0.2/opennms-services/src/test/java/org/opennms/netmgt/collectd/CollectableServiceTest.java <==
/*
 * Licensed to The OpenNMS Group, Inc (TOG) under one or more
 * contributor license agreements.  See the LICENSE.md file
 * distributed with this work for additional information
 * regarding copyright ownership.
 *
 * TOG licenses this file to You under the GNU Affero General
 * Public License Version 3 (the "License") or (at your option)
 * any later version.  You may not use this file except in
 * compliance with the License.  You may obtain a copy of the
 * License at:
 *
 *      https://www.gnu.org/licenses/agpl-3.0.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.  See the License for the specific
 * language governing permissions and limitations under the
 * License.
 */
package org.opennms.netmgt.collectd;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.RETURNS_DEEP_STUBS;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.jrobin.core.RrdDb;
import org.jrobin.core.RrdException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.opennms.core.test.MockLogAppender;
import org.opennms.netmgt.collectd.Collectd.SchedulingCompletedFlag;
import org.opennms.netmgt.collection.api.AttributeType;
import org.opennms.netmgt.collection.api.CollectionAgent;
import org.opennms.netmgt.collection.api.CollectionException;
import org.opennms.netmgt.collection.api.CollectionInitializationException;
import org.opennms.netmgt.collection.api.CollectionSet;
import org.opennms.netmgt.collection.api.ServiceParameters;
import org.opennms.netmgt.collection.core.CollectionSpecification;
import org.opennms.netmgt.collection.persistence.rrd.RrdPersisterFactory;
import org.opennms.netmgt.collection.support.builder.CollectionSetBuilder;
import org.opennms.netmgt.collection.support.builder.NodeLevelResource;
import org.opennms.netmgt.dao.api.IpInterfaceDao;
import org.opennms.netmgt.dao.api.ResourceStorageDao;
import org.opennms.netmgt.dao.mock.MockEventIpcManager;
import org.opennms.netmgt.events.api.EventIpcManagerFactory;
import org.opennms.netmgt.model.OnmsIpInterface;
import org.opennms.netmgt.rrd.RrdRepository;
import org.opennms.netmgt.rrd.RrdStrategy;
import org.opennms.netmgt.rrd.jrobin.JRobinRrdStrategy;
import org.opennms.netmgt.scheduler.Scheduler;
import org.opennms.netmgt.snmp.InetAddrUtils;
import org.opennms.netmgt.threshd.api.ThresholdingService;
import org.opennms.test.FileAnticipator;
import org.springframework.transaction.PlatformTransactionManager;

public class CollectableServiceTest {

    private CollectionSpecification spec;
    private Scheduler scheduler;
    private CollectableService service;

    private File snmpDirectory;
    private FileAnticipator fileAnticipator;
    private RrdStrategy<?, ?> rrdStrategy;

    @Before
    public void setUp() throws Exception {
        MockLogAppender.setupLogging();

        rrdStrategy = new JRobinRrdStrategy();
        fileAnticipator = new FileAnticipator();

        MockEventIpcManager mockEventIpcManager = new MockEventIpcManager();
        EventIpcManagerFactory.setIpcManager(mockEventIpcManager);
    }

    @After
    public void tearDown() {
        System.clearProperty(CollectableService.STRICT_INTERVAL_SYS_PROP);
        System.clearProperty(CollectableService.USE_COLLECTION_START_TIME_SYS_PROP);

        MockLogAppender.assertNoErrorOrGreater();
        fileAnticipator.deleteExpected();
        fileAnticipator.tearDown();
    }

    /**
     * Verifies that CollectableService is rescheduled according
     * to the specification's interval when the "strict interval" property is disabled
     * (which is by default).
     */
    @Test
    public void collectUsingSpecifiedInterval() throws CollectionInitializationException, CollectionException, IOException {

        System.setProperty(CollectableService.STRICT_INTERVAL_SYS_PROP, Boolean.FALSE.toString());
        createCollectableService();

        Long serviceIntervalInMs = 300 * 1000L;
        Long collectionDelayInMs = 1 * 1000L;
        when(spec.getInterval()).thenReturn(serviceIntervalInMs);
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInMs);
                return null;
            }
        });

        ArgumentCaptor<Long> intervalCaptor = ArgumentCaptor.forClass(Long.class);
        service.run();
        verify(scheduler, times(1)).schedule(intervalCaptor.capture(), any());
        assertEquals(serviceIntervalInMs, intervalCaptor.getValue());
    }

    /**
     * Verifies that collection time is subtracted from the interval
     * when then "strict interval" property is enabled.
     */
    @Test
    public void collectUsingStrictInterval() throws CollectionInitializationException, CollectionException, IOException {

        createCollectableService();

        Long serviceIntervalInMs = 2000L;
        Long collectionDelayInMs = 500L;
        when(spec.getInterval()).thenReturn(serviceIntervalInMs);
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInMs);
                return null;
            }
        });

        // Run the CollectableService and verify that the collection time is
        // subtracted from the interval when the "strict interval" property is enabled
        ArgumentCaptor<Long> intervalCaptor = ArgumentCaptor.forClass(Long.class);
        service.run();
        verify(scheduler, times(1)).schedule(intervalCaptor.capture(), any());

        Long upperBound = serviceIntervalInMs - collectionDelayInMs;
        assertTrue(String.format("Expected the interval to be less than %d, but was %d",
                upperBound, intervalCaptor.getValue()), intervalCaptor.getValue() <= upperBound);

        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(2 * serviceIntervalInMs);
                return null;
            }
        });

        service.run();
        verify(scheduler, times(2)).schedule(intervalCaptor.capture(), any());
        assertEquals(Long.valueOf(0), intervalCaptor.getValue());
    }

    /**
     * Validates that we can successfully wrap collection sets with a custom time-keeper,
     * allowing us to override the timestamp of the attributes within the collection set.
     */
    @Test
    public void canWrapResourcesWithTimekeeper() throws CollectionInitializationException, CollectionException, IOException, RrdException {
        System.setProperty(CollectableService.USE_COLLECTION_START_TIME_SYS_PROP, Boolean.TRUE.toString());
        createCollectableService();

        long collectionDelayInSecs = 2;
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInSecs * 1000);
                CollectionAgent agent = (CollectionAgent)invocation.getArguments()[0];
                NodeLevelResource nodeResource = new NodeLevelResource(agent.getNodeId());
                return new CollectionSetBuilder(agent)
                    .withNumericAttribute(nodeResource, "mibGroup", "myCounter", 1000, AttributeType.COUNTER)
                    .build();
            }
        });

        File nodeDir = fileAnticipator.expecting(getSnmpRrdDirectory(), "1");
        File jrbFile = fileAnticipator.expecting(nodeDir, "myCounter" + rrdStrategy.getDefaultFileExtension());
        fileAnticipator.expecting(nodeDir, "myCounter" + ".meta");

        long beforeInMs = System.currentTimeMillis();
        service.run();
        long afterInMs = System.currentTimeMillis();
        // Quick sanity check
        assertTrue(String.format("Delay was not succesfully applied (delay was %d).",
                beforeInMs - afterInMs), afterInMs - beforeInMs >= collectionDelayInSecs * 1000);

        // Verify the last update time match the start of the collection time
        RrdDb rrdDb = new RrdDb(jrbFile);
        long lastUpdateTimeInSecs = rrdDb.getLastUpdateTime();
        long beforeInSecs = Math.floorDiv(beforeInMs, 1000);
        long afterInSecs = Math.floorDiv(afterInMs, 1000) + 1;

        assertTrue("Last update was before the collector was invoked!",
                lastUpdateTimeInSecs >= beforeInSecs);
        assertTrue("Last update was too long after the collector was invoked!",
                lastUpdateTimeInSecs < (afterInSecs - (collectionDelayInSecs / 2d)));
    }

    private void createCollectableService() throws CollectionInitializationException, IOException {
        // Mock it all!
        OnmsIpInterface iface = mock(OnmsIpInterface.class, RETURNS_DEEP_STUBS);
        IpInterfaceDao ifaceDao = mock(IpInterfaceDao.class);
        spec = mock(CollectionSpecification.class);
        scheduler = mock(Scheduler.class);
        SchedulingCompletedFlag schedulingCompletedFlag = mock(SchedulingCompletedFlag.class);
        PlatformTransactionManager transMgr = mock(PlatformTransactionManager.class);
        RrdPersisterFactory persisterFactory = new RrdPersisterFactory();
        persisterFactory.setRrdStrategy(rrdStrategy);
        ResourceStorageDao resourceStorageDao = mock(ResourceStorageDao.class);

        // Disable thresholding
        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("thresholding-enabled", Boolean.FALSE.toString());
        ServiceParameters params = new ServiceParameters(paramsMap);

        when(iface.getNode().getId()).thenReturn(1);
        when(spec.getServiceParameters()).thenReturn(params);
        when(spec.getRrdRepository(any())).thenReturn(createRrdRepository());
        when(ifaceDao.load(any())).thenReturn(iface);
        when(iface.getIpAddress()).thenReturn(InetAddrUtils.getLocalHostAddress());

        ThresholdingService mockThresholdingService = mock(ThresholdingService.class, RETURNS_DEEP_STUBS);

        service = new CollectableService(iface, ifaceDao, spec, scheduler, schedulingCompletedFlag, transMgr, persisterFactory, mockThresholdingService);
    }

    private RrdRepository createRrdRepository() throws IOException {
        RrdRepository repository = new RrdRepository();
        repository.setRrdBaseDir(getSnmpRrdDirectory());
        repository.setHeartBeat(600);
        repository.setStep(300);
        repository.setRraList(Collections.singletonList("RRA:AVERAGE:0.5:1:100"));
        return repository;
    }

    private File getSnmpRrdDirectory() throws IOException {
        if (snmpDirectory == null) {
            snmpDirectory = fileAnticipator.tempDir("snmp");
        }
        return snmpDirectory;
    }
}

==> opennms-35.0.3.tar.gz: opennms-35.0.3/opennms-services/src/test/java/org/opennms/netmgt/collectd/CollectableServiceTest.java <==
/*
 * Licensed to The OpenNMS Group, Inc (TOG) under one or more
 * contributor license agreements.  See the LICENSE.md file
 * distributed with this work for additional information
 * regarding copyright ownership.
 *
 * TOG licenses this file to You under the GNU Affero General
 * Public License Version 3 (the "License") or (at your option)
 * any later version.  You may not use this file except in
 * compliance with the License.  You may obtain a copy of the
 * License at:
 *
 *      https://www.gnu.org/licenses/agpl-3.0.txt
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.  See the License for the specific
 * language governing permissions and limitations under the
 * License.
 */
package org.opennms.netmgt.collectd;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.RETURNS_DEEP_STUBS;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.jrobin.core.RrdException;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.opennms.core.test.MockLogAppender;
import org.opennms.netmgt.collectd.Collectd.SchedulingCompletedFlag;
import org.opennms.netmgt.collection.api.AttributeType;
import org.opennms.netmgt.collection.api.CollectionAgent;
import org.opennms.netmgt.collection.api.CollectionException;
import org.opennms.netmgt.collection.api.CollectionInitializationException;
import org.opennms.netmgt.collection.api.CollectionSet;
import org.opennms.netmgt.collection.api.ServiceParameters;
import org.opennms.netmgt.collection.core.CollectionSpecification;
import org.opennms.netmgt.collection.persistence.rrd.RrdPersisterFactory;
import org.opennms.netmgt.collection.support.builder.CollectionSetBuilder;
import org.opennms.netmgt.collection.support.builder.NodeLevelResource;
import org.opennms.netmgt.dao.api.IpInterfaceDao;
import org.opennms.netmgt.dao.api.ResourceStorageDao;
import org.opennms.netmgt.dao.mock.MockEventIpcManager;
import org.opennms.netmgt.events.api.EventIpcManagerFactory;
import org.opennms.netmgt.model.OnmsIpInterface;
import org.opennms.netmgt.rrd.RrdRepository;
import org.opennms.netmgt.rrd.RrdStrategy;
import org.opennms.netmgt.rrd.util.RrdConvertUtils;
import org.opennms.netmgt.rrd.rrdtool.MultithreadedJniRrdStrategy;
import org.opennms.netmgt.scheduler.Scheduler;
import org.opennms.netmgt.snmp.InetAddrUtils;
import org.opennms.netmgt.threshd.api.ThresholdingService;
import org.opennms.test.FileAnticipator;
import org.springframework.transaction.PlatformTransactionManager;

public class CollectableServiceTest {

    private CollectionSpecification spec;
    private Scheduler scheduler;
    private CollectableService service;

    private File snmpDirectory;
    private FileAnticipator fileAnticipator;
    private RrdStrategy<?, ?> rrdStrategy;

    @Before
    public void setUp() throws Exception {
        System.setProperty("rrd.binary", "rrdtool");

        MockLogAppender.setupLogging();

        rrdStrategy = new MultithreadedJniRrdStrategy();
        fileAnticipator = new FileAnticipator();

        MockEventIpcManager mockEventIpcManager = new MockEventIpcManager();
        EventIpcManagerFactory.setIpcManager(mockEventIpcManager);
    }

    @After
    public void tearDown() {
        System.clearProperty(CollectableService.STRICT_INTERVAL_SYS_PROP);
        System.clearProperty(CollectableService.USE_COLLECTION_START_TIME_SYS_PROP);

        MockLogAppender.assertNoErrorOrGreater();
        fileAnticipator.deleteExpected();
        fileAnticipator.tearDown();
    }

    /**
     * Verifies that CollectableService is rescheduled according
     * to the specification's interval when the "strict interval" property is disabled
     * (which is by default).
     */
    @Test
    public void collectUsingSpecifiedInterval() throws CollectionInitializationException, CollectionException, IOException {

        System.setProperty(CollectableService.STRICT_INTERVAL_SYS_PROP, Boolean.FALSE.toString());
        createCollectableService();

        Long serviceIntervalInMs = 300 * 1000L;
        Long collectionDelayInMs = 1 * 1000L;
        when(spec.getInterval()).thenReturn(serviceIntervalInMs);
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInMs);
                return null;
            }
        });

        ArgumentCaptor<Long> intervalCaptor = ArgumentCaptor.forClass(Long.class);
        service.run();
        verify(scheduler, times(1)).schedule(intervalCaptor.capture(), any());
        assertEquals(serviceIntervalInMs, intervalCaptor.getValue());
    }

    /**
     * Verifies that collection time is subtracted from the interval
     * when then "strict interval" property is enabled.
     */
    @Test
    public void collectUsingStrictInterval() throws CollectionInitializationException, CollectionException, IOException {

        createCollectableService();

        Long serviceIntervalInMs = 2000L;
        Long collectionDelayInMs = 500L;
        when(spec.getInterval()).thenReturn(serviceIntervalInMs);
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInMs);
                return null;
            }
        });

        // Run the CollectableService and verify that the collection time is
        // subtracted from the interval when the "strict interval" property is enabled
        ArgumentCaptor<Long> intervalCaptor = ArgumentCaptor.forClass(Long.class);
        service.run();
        verify(scheduler, times(1)).schedule(intervalCaptor.capture(), any());

        Long upperBound = serviceIntervalInMs - collectionDelayInMs;
        assertTrue(String.format("Expected the interval to be less than %d, but was %d",
                upperBound, intervalCaptor.getValue()), intervalCaptor.getValue() <= upperBound);

        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(2 * serviceIntervalInMs);
                return null;
            }
        });

        service.run();
        verify(scheduler, times(2)).schedule(intervalCaptor.capture(), any());
        assertEquals(Long.valueOf(0), intervalCaptor.getValue());
    }

    /**
     * Validates that we can successfully wrap collection sets with a custom time-keeper,
     * allowing us to override the timestamp of the attributes within the collection set.
     */
    @Test
    public void canWrapResourcesWithTimekeeper() throws CollectionInitializationException, CollectionException, IOException, RrdException {
        System.setProperty(CollectableService.USE_COLLECTION_START_TIME_SYS_PROP, Boolean.TRUE.toString());
        createCollectableService();

        long collectionDelayInSecs = 2;
        when(spec.collect(any())).then(new Answer<CollectionSet>() {
            @Override
            public CollectionSet answer(InvocationOnMock invocation) throws InterruptedException {
                Thread.sleep(collectionDelayInSecs * 1000);
                CollectionAgent agent = (CollectionAgent)invocation.getArguments()[0];
                NodeLevelResource nodeResource = new NodeLevelResource(agent.getNodeId());
                return new CollectionSetBuilder(agent)
                    .withNumericAttribute(nodeResource, "mibGroup", "myCounter", 1000, AttributeType.COUNTER)
                    .build();
            }
        });

        File nodeDir = fileAnticipator.expecting(getSnmpRrdDirectory(), "1");
        File rrdFile = fileAnticipator.expecting(nodeDir, "myCounter" + rrdStrategy.getDefaultFileExtension());
        fileAnticipator.expecting(nodeDir, "myCounter" + ".meta");

        long beforeInMs = System.currentTimeMillis();
        service.run();
        long afterInMs = System.currentTimeMillis();
        // Quick sanity check
        assertTrue(String.format("Delay was not succesfully applied (delay was %d).",
                beforeInMs - afterInMs), afterInMs - beforeInMs >= collectionDelayInSecs * 1000);

        // Verify the last update time match the start of the collection time
        var rrd = RrdConvertUtils.dumpRrd(rrdFile);
        long lastUpdateTimeInSecs = rrd.getLastUpdate();
        long beforeInSecs = Math.floorDiv(beforeInMs, 1000);
        long afterInSecs = Math.floorDiv(afterInMs, 1000) + 1;

        assertTrue("Last update was before the collector was invoked!",
                lastUpdateTimeInSecs >= beforeInSecs);
        assertTrue("Last update was too long after the collector was invoked!",
                lastUpdateTimeInSecs < (afterInSecs - (collectionDelayInSecs / 2d)));
    }

    private void createCollectableService() throws CollectionInitializationException, IOException {
        // Mock it all!
        OnmsIpInterface iface = mock(OnmsIpInterface.class, RETURNS_DEEP_STUBS);
        IpInterfaceDao ifaceDao = mock(IpInterfaceDao.class);
        spec = mock(CollectionSpecification.class);
        scheduler = mock(Scheduler.class);
        SchedulingCompletedFlag schedulingCompletedFlag = mock(SchedulingCompletedFlag.class);
        PlatformTransactionManager transMgr = mock(PlatformTransactionManager.class);
        RrdPersisterFactory persisterFactory = new RrdPersisterFactory();
        persisterFactory.setRrdStrategy(rrdStrategy);
        ResourceStorageDao resourceStorageDao = mock(ResourceStorageDao.class);

        // Disable thresholding
        Map<String, Object> paramsMap = new HashMap<>();
        paramsMap.put("thresholding-enabled", Boolean.FALSE.toString());
        ServiceParameters params = new ServiceParameters(paramsMap);

        when(iface.getNode().getId()).thenReturn(1);
        when(spec.getServiceParameters()).thenReturn(params);
        when(spec.getRrdRepository(any())).thenReturn(createRrdRepository());
        when(ifaceDao.load(any())).thenReturn(iface);
        when(iface.getIpAddress()).thenReturn(InetAddrUtils.getLocalHostAddress());

        ThresholdingService mockThresholdingService = mock(ThresholdingService.class, RETURNS_DEEP_STUBS);

        service = new CollectableService(iface, ifaceDao, spec, scheduler, schedulingCompletedFlag, transMgr, persisterFactory, mockThresholdingService);
    }

    private RrdRepository createRrdRepository() throws IOException {
        RrdRepository repository = new RrdRepository();
        repository.setRrdBaseDir(getSnmpRrdDirectory());
        repository.setHeartBeat(600);
        repository.setStep(300);
        repository.setRraList(Collections.singletonList("RRA:AVERAGE:0.5:1:100"));
        return repository;
    }

    private File getSnmpRrdDirectory() throws IOException {
        if (snmpDirectory == null) {
            snmpDirectory = fileAnticipator.tempDir("snmp");
        }
        return snmpDirectory;
    }
}
